# -*- coding: utf-8 -*-
import zipfile
import sys
from .base import CompressFieldFile
try:
    import zlib
    compression = zipfile.ZIP_DEFLATED
except ImportError:
    compression = zipfile.ZIP_STORED


class ZipCompressFieldFile(CompressFieldFile):
    compress_ext = 'zip'

    def compress_content(self):
        compress_file_fullname = self.compress_full_name

        # to work with py2.6 or lower
        if sys.version_info < (2, 7):
            compress_file_fullname = open(compress_file_fullname, 'w+b')

        if not zipfile.is_zipfile(self.file.name):
            ziped = zipfile.ZipFile(compress_file_fullname, 'w', compression=compression)
            ziped.write(self.file.name, self.base_name)
            ziped.close()
            return ziped



# -*- coding: utf-8 -*-
try:
    from celery.task import task
except ImportError:
    task = None

import os
from django.conf import settings
from django.db.models.fields.files import FieldFile

FILE_COMPRESS_DELETE_OLD_FILE = getattr(settings, 'FILE_COMPRESS_DELETE_OLD_FILE', True)


class CompressFieldFile(FieldFile):
    compress_ext = None

    def _is_compressed(self):
        basename, ext = os.path.splitext(self.name)
        compress_ext = '.' + self.compress_ext
        return compress_ext == ext
    is_compressed = property(_is_compressed)

    def _base_name(self):
        return os.path.basename(self.file.name)
    base_name = property(_base_name)

    def _compress_name(self):
        if not hasattr(self, '_avaliable_compress_name'):
            basename, ext = os.path.splitext(self.name)
            compress_name = basename + ('.' + self.compress_ext)
            self._avaliable_compress_name = self.storage.get_available_name(compress_name)
        return self._avaliable_compress_name

    compress_name = property(_compress_name)

    def _compress_full_name(self):
        return os.path.join(self.storage.location, self.compress_name)

    compress_full_name = property(_compress_full_name)

    def get_available_name(self):
        return self.storage.get_available_name(self.compress_name)

    def compress_content(self):
        """
        Method to change for new implementations
        """
        raise NotImplementedError()

    def _update_filefield_name(self, delete_old_file=True):
        # update field value
        old_name = self.name

        setattr(self.instance, self.field.name, self.compress_name)
        self.instance.save()

        if delete_old_file:
            self.file.close()
            self.storage.delete(old_name)

    def compress_wrapper(self, delete_old_file):
        self.compress_content()
        self._update_filefield_name(delete_old_file=delete_old_file)
        return True

    def compress(self, async=True, delete_old_file=FILE_COMPRESS_DELETE_OLD_FILE):
        if self.is_compressed:
            return u'This file is alredy compress'

        if async and task:
            from .tasks import task_compress_wrapper
            wrapper = task_compress_wrapper.delay(self.instance, self.field.name, delete_old_file)
        else:
            wrapper = CompressFieldFile.compress_wrapper(self, delete_old_file)

        return wrapper



# -*- coding: utf-8 -*-
try:
    from celery.task import task
except ImportError:
    task = None

from django.core.cache import cache
from django.conf import settings
LOCK_EXPIRE = 60 * 5

FILE_COMPRESS_QUEUE = getattr(settings, 'FILE_COMPRESS_QUEUE', 'Celery')


# cache.add fails if if the key already exists
def acquire_lock(lock_id):
    return cache.add(lock_id, 'true', LOCK_EXPIRE)


# memcache delete is very slow, but we have to use it to take
# advantage of using add() for atomic locking
def release_lock(lock_id):
    return cache.delete(lock_id)


@task(serializer='pickle', queue=FILE_COMPRESS_QUEUE)
def task_compress_wrapper(instance, field, delete_old_file):
    lock_id = '{0}-io-lock-{1}'.format(instance.__class__.__name__, instance.id)

    if acquire_lock(lock_id):
        instance_field = getattr(instance, field)
        instance_field.compress(async=False, delete_old_file=delete_old_file)
        release_lock(lock_id)
        return True

    # task is locked by IO
    print('IO Lock Task')
    return False



# -*- coding: utf-8 -*-
from .fieldfiles import ZipCompressFieldFile
from django.db import models


class ZipFileField(models.FileField):
    """
    ZipField - auto zip content after file was saved on server
    """
    attr_class = ZipCompressFieldFile


try:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^compress_storage\.models\.(ZipFileField)"])
except ImportError:
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

os.environ['DJANGO_SETTINGS_MODULE'] = 'example.test_settings'

import compress_storage

install_requires = [
    'django>=1.2',
]


def readme():
    try:
        os.system('pandoc --from=markdown --to=rst README.md -o README.rst')
        with open('README.rst') as f:
            return f.read()
    except:
        return 'Automantic compress files after upload'


setup(name='django-compress-storage',
      url='https://github.com/valdergallo/django-compress-storage',
      download_url='https://github.com/valdergallo/django-compress-storage/tarball/v%s/' % compress_storage.__version__,
      author="valdergallo",
      author_email='valdergallo@gmail.com',
      keywords=['django', 'compress', 'field', 'zip', 'tar', 'gzip'],
      description='Automantic compress files after upload',
      long_description=readme(),
      license='FREEBSD',
      classifiers=[
          'Framework :: Django',
          'Operating System :: OS Independent',
          'Topic :: Utilities',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.3',
      ],
      include_package_data=True,
      version=compress_storage.__version__,
      install_requires=install_requires,
      test_suite="runtest.runtests",
      packages=find_packages(where='.',
                             exclude=('*test*', '*example*', 'runtest')),
)
